/*
 * iGeom by LInE
 * Geometric Object: Segment
 * Manage intersection of Segment with: Sements, Circumference
 * www.matematica.br/igeom
 * .app/components/line-segment-component/models/line-segment-model.js
 * @version 2020/11/02: Implemented Line instersection
 */

import { GeometricObject } from "../../../core/models/objects/geometric-object";
import { PointModel } from "../../point-component/models/point-model";
import { IntersectionModel } from "../../intersection-component/models/intersection-model";
import { ELEMENTS_CLASS } from "../../../core/enums/elements-class-enum";

export class MiddlePointModel extends PointModel {

  constructor (pointA, pointB, label, id) {
  //D console.trace();
  try {
    super(id);
    this.pointA = pointA;
    this.pointB = pointB;
    const middlePoint = this.getMiddlePoint();
    this.selectable = true; // para KonvaOjbect: object.attrs.selectable
    console.log("middle-point-model.js!constructor(.): this.id=" + this.id + ", this.selectable=" + this.selectable); //D
    //r this.middlePoint = middlePoint; // used by 'middle-point-drawer.js!drawByStates(konvaObject)
    this.posX = middlePoint.posX;
    this.posY = middlePoint.posY;
    this.setLabel(label);
    super.setClass(ELEMENTS_CLASS.MIDDLE_POINT);
    //this.definitions.push(this.pointA); // to be used in GEO file DEFINITION, see app/core/models/application/actions/action.js
    //this.definitions.push(this.pointB);
    this.definitions = this.getDefinitions();
    this.color = 16711936; //-16711936 = verde; // "#f54260"
    } catch (error) {
      console.log("middle-point-model.js!constructor(.): this.id=" + this.id + ", error " + error);
      console.log("middle-point-model.js!constructor(.): error A=" + GeometricObject.printMap_go(pointA) + ", B=" + GeometricObject.printMap_go(pointB));
      }
    }

  // To be used in GEO file DEFINITION, see app/core/models/application/actions/action.js
  getDefinitions () {
    try { //D //leo
      console.log("middle-point-model.js!getDefinitions(); aqui!"); //D
      return [{ id: this.pointA.id }, { id: this.pointB.id } ]; // , { id: this.index + 1 }, { id: this.visible ? 1 : 0 }
    } catch (e) { console.log("intersection-model.js: getDefinitions(): erro!"); }
    }


  getMiddlePoint () {
    return MiddlePointModel.getMiddlePointPos(this.pointA, this.pointB);
    }

  static getMiddlePointPos (pointA, pointB) {
    const x = (pointA.posX + pointB.posX) / 2;
    const y = (pointA.posY + pointB.posY) / 2;
    return { posX: x, posY: y };
    }

  // Help: search object in more stable structure
  static findOD (vecIdObj, id) {
    var tam = vecIdObj.length, ii;
    for (ii=0; ii<tam; ii++)
      if (vecIdObj[ii].id === id) return ii;
    return -1; // Error! Not found!
    }

  // In reading GEO file, need to transform a Map in GeometricObject
  // @see app/core/parser/file-parser.js
  // @see app/core/parser/parser-orchestrator.js
  static do (map, vecIdObj, list) {
    // 2023/07/18: "vecIdObj" is element of Array with { id : only_id, obj : parsed_object } (build in app/core/application/header-menu.js!onFileChanged()
    //             each element is similar to: [id=1] [obj=[object Map]]
    var id="-1", pointAId="-1", pointBId="-1", pointA="-1", pointB="-1", label="-1", index_pointA, index_pointB;
    var aux2 = JSON.stringify(Object.values(map)), aux3 = JSON.stringify(Object.values(vecIdObj)); //D GeometricObject.printMap_go(map,"map") GeometricObject.printMap_go(list,"list")
    var aux = "", aux4 = ""; //D Object.keys
    console.log("middle-point-model.js!map(.): id=" + id + "\n* map=" + aux2 + "\n* vecIdObj=" + aux3); //D
    try {
      id = map.get("id"); aux4 += "1,";
      pointAId = map.get("param")[0]; aux4 += "2,";
      pointBId = map.get("param")[1]; aux4 += "3,";
      // pointA = list.find(x => x.id === pointAId); // find in "list" the pointA
      // pointB = list.find(x => x.id === pointBId); // find in "list" the pointB
      index_pointA = MiddlePointModel.findOD(vecIdObj, pointAId); // get the index of vecIdObj[ii].id == pointAId
      index_pointB = MiddlePointModel.findOD(vecIdObj, pointBId); // get the index of vecIdObj[ii].id == pointBId
      aux4 += "4,";
      pointA = list[index_pointA]; // pointA = vecIdObj[index_pointA].obj; aux4 += "5,";
      pointB = list[index_pointB]; // pointB = vecIdObj[index_pointB].obj; aux4 += "6,";
      label = map.get("label")[0]; aux4 += "7,";
    } catch (error) {
      if (!pointA.id) pointA.id = -1; // security (inscrease error proof)
      if (!pointB.id) pointB.id = -1; // security (inscrease error proof)
      aux = "\nError: id=" + id + " : " + aux4;
      }
    if (!pointA.id) aux += " - index=" + index_pointA + ", pointA.id=<>="+JSON.stringify(pointA); else aux += " - index=" + index_pointA + ", pointA.id=" + pointA.id;
    if (!pointB.id) aux += " - index=" + index_pointB + ", pointB.id=<>="+JSON.stringify(pointB); else aux += " - index=" + index_pointB + ", pointB.id=" + pointB.id;
    console.log("middle-point-model.js!map(.): id=" + id + ", pointA.id=" + pointAId + "=" + pointA.id + ", pointB.id=" + pointBId + "=" + pointB.id + ", label=" + label + aux); //D
    return new MiddlePointModel(pointA, pointB, label, id);
    }

  }